@@ -1,23 +1,87 @@
 Nitrate.TestCases = {};
+Nitrate.TestCases.Search = {};
 Nitrate.TestCases.List = {};
-Nitrate.TestCases.AdvanceList = {};
+Nitrate.TestCases.AdvancedSearch = {};
 Nitrate.TestCases.Details = {};
 Nitrate.TestCases.Create = {};
 Nitrate.TestCases.Edit = {};
 Nitrate.TestCases.Clone = {};
 
-Nitrate.TestCases.AdvanceList.on_load = function () {
-  jQ('#testcases_table :checkbox').on('change', function () {
-    jQ('#case_advance_printable').prop(
-      'disabled', jQ('#testcases_table tbody :checkbox:checked').length === 0
-    );
-  });
+Nitrate.TestCases.SearchResultTableSettings = Object.assign({}, Nitrate.DataTable.commonSettings, {
 
-  jQ('#testcases_table tbody tr td:nth-child(2)').shiftcheckbox({
-    checkboxSelector: ':checkbox',
-    selectAll: '#testcases_table .js-select-all'
-  });
+  // By default, cases are sorted by create_date in desc order.
+  // It is equal to set the pk column in the DataTable initialization.
+  aaSorting: [[ 2, 'desc' ]],
 
+  oLanguage: {
+    sEmptyTable: 'No cases found.'
+  },
+
+  aoColumns: [
+    {'bSortable': false, 'sClass': 'expandable'},   // the expand/collapse control
+    {'bSortable': false},                           // case selector checkbox
+    null,                                           // ID
+    {'sClass': 'expandable'},                       // Summary
+    null,                                           // Author
+    null,                                           // Default tester
+    null,                                           // Automated
+    null,                                           // Status
+    null,                                           // Category
+    null,                                           // Priority
+    null,                                           // Created on
+  ],
+
+  fnDrawCallback: function () {
+    // Add the ajax_loading row to show case details
+    this.fnGetNodes().forEach(function (tr) {
+      let caseId = jQ(tr).data('pk');
+      jQ(tr).after(jQ(
+        '<tr class="case_content hide" style="display: none;">' +
+        '<td colspan="' + tr.children.length + '">' +
+        '<div id="id_loading_' + caseId + '" class="ajax_loading"></div>' +
+        '</td>' +
+        '</tr>'
+      ));
+    });
+
+    jQ('#testcases_table tbody tr td:nth-child(2)').shiftcheckbox({
+      checkboxSelector: ':checkbox',
+      selectAll: '#testcases_table .js-select-all'
+    });
+
+    jQ('#testcases_table tbody :checkbox').on('change', function () {
+      Nitrate.TestCases.Search.setActionButtonsStatus(
+        jQ('#testcases_table tbody :checkbox:checked').length === 0
+      );
+    });
+
+    jQ('.expandable').on('click', function () {
+      let c = jQ(this).parent()[0]; // Container
+      let cContainer = jQ(c).next()[0]; // Content Containers
+      let caseId = parseInt(jQ(c).find(':checkbox').prop('value'));
+
+      toggleTestCasePane({
+        case_id: caseId,
+        casePaneContainer: jQ(cContainer)
+      });
+      toggleExpandArrow({
+        caseRowContainer: jQ(c),
+        expandPaneContainer: jQ(cContainer)
+      });
+    });
+  },
+
+  fnInfoCallback: function (oSettings, iStart, iEnd, iMax, iTotal, sPre) {
+    return 'Showing ' + (iEnd - iStart + 1) + ' of ' + iTotal + ' cases';
+  }
+});
+
+/**
+ * Initialize test cases search result table and associated action buttons.
+ *
+ * @param {string} searchEndpoint - the search endpoint.
+ */
+Nitrate.TestCases.Search.initializeSearchResult = function (searchEndpoint) {
   jQ('#id_blind_all_link').on('click', function () {
     if (!jQ('div[id^="id_loading_"]').length) {
       jQ(this).removeClass('locked');
@@ -38,32 +102,41 @@ Nitrate.TestCases.AdvanceList.on_load = function () {
     }
   });
 
-  jQ('.expandable').on('click', function () {
-    let c = jQ(this).parent()[0]; // Container
-    let cContainer = jQ(c).next()[0]; // Content Containers
-    let caseId = jQ(c).find('input[name="case"]')[0].value;
-
-    toggleTestCasePane({
-      case_id: caseId,
-      casePaneContainer: jQ(cContainer)
-    });
-    toggleExpandArrow({
-      caseRowContainer: jQ(c),
-      expandPaneContainer: jQ(cContainer)
-    });
+  jQ('#testcases_table .js-select-all').on('change', function () {
+    Nitrate.TestCases.Search.setActionButtonsStatus(!this.checked);
   });
 
   if (window.location.hash === '#expandall') {
     blinddownAllCases();
   }
 
-  let listParams = Nitrate.TestCases.List.Param;
-  jQ('#case_advance_printable').on('click', function () {
-    postToURL(listParams.case_printable, Nitrate.Utils.formSerialize(this.form));
+  let urls = Nitrate.TestCases.Search.ActionUrls;
+
+  jQ('.js-printable-cases').on('click', function () {
+    postToURL(urls.printableCasesUrl, Nitrate.Utils.formSerialize(this.form));
   });
-  jQ('#export_selected_cases').on('click', function () {
-    postToURL(listParams.case_export, Nitrate.Utils.formSerialize(this.form));
+
+  jQ('.js-export-cases').on('click', function () {
+    postToURL(urls.exportCasesUrl, Nitrate.Utils.formSerialize(this.form));
   });
+
+  jQ('#testcases_table').dataTable(
+    Object.assign({}, Nitrate.TestCases.SearchResultTableSettings, {
+      iDeferLoading: Nitrate.TestCases.Search.numberOfCases,
+      sAjaxSource: searchEndpoint + window.location.search,
+    })
+  );
+};
+
+/**
+ * Set action buttons' status
+ *
+ * @param {boolean} disabled - true for disable, otherwise enable them.
+ */
+Nitrate.TestCases.Search.setActionButtonsStatus = function (disabled) {
+  jQ('.js-clone-cases').prop('disabled', disabled);
+  jQ('.js-export-cases').prop('disabled', disabled);
+  jQ('.js-printable-cases').prop('disabled', disabled);
 };
 
 Nitrate.TestCases.List.on_load = function () {
@@ -84,104 +157,11 @@ Nitrate.TestCases.List.on_load = function () {
     ]
   );
 
-  /* Event handlers of case expansion/collapse */
+  Nitrate.TestCases.Search.initializeSearchResult('/cases/search/');
+};
 
-  jQ('#id_blind_all_link').on('click', function () {
-    if (!jQ('div[id^="id_loading_"]').length) {
-      jQ(this).removeClass('locked');
-    }
-    if (jQ(this).is('.locked')) {
-      //To disable the 'expand all' until all case runs are expanded.
-      return false;
-    } else {
-      jQ(this).addClass('locked');
-      let element = jQ(this).children()[0];
-      if (jQ(element).is('.collapse-all')) {
-        this.title = 'Collapse all cases';
-        blinddownAllCases(element);
-      } else {
-        this.title = 'Expand all cases';
-        blindupAllCases(element);
-      }
-    }
-  });
-
-  jQ('#testcases_table tbody tr td.expandable').on('click', function () {
-    let tr = jQ(this).parent();
-    let caseRowContainer = tr;
-    let caseId = caseRowContainer.find('input[name="case"]').prop('value');
-    let detailTd = '<tr class="case_content hide" style="display: none;"><td colspan="11">' +
-      '<div id="id_loading_' + caseId + '" class="ajax_loading"></div></td></tr>';
-    if (!caseRowContainer.next().hasClass('hide')) {
-      caseRowContainer.after(detailTd);
-    }
-
-    toggleTestCasePane({
-      case_id: caseId,
-      casePaneContainer: tr.next()
-    });
-    toggleExpandArrow({
-      caseRowContainer: tr,
-      expandPaneContainer: tr.next()
-    });
-  });
-
-  if (window.location.hash === '#expandall') {
-    blinddownAllCases();
-  }
-
-  /* Initialize cases search result table and relative controls */
-
-  jQ('#testcases_table').dataTable({
-    'iDisplayLength': 20,
-    'sPaginationType': 'full_numbers',
-    'bFilter': false,
-    'bLengthChange': false,
-    'aaSorting': [[ 2, 'desc' ]],
-    'bProcessing': true,
-    'bServerSide': true,
-    'sAjaxSource': '/cases/ajax/' + this.window.location.search,
-    'aoColumns': [
-      {'bSortable': false, 'sClass': 'expandable'},
-      {'bSortable': false},
-      {'sType': 'html', 'sClass': 'expandable'},
-      {'sType': 'html', 'sClass': 'expandable'},
-      {'sType': 'html', 'sClass': 'expandable'},
-      {'sClass': 'expandable'},
-      {'sClass': 'expandable'},
-      {'sClass': 'expandable'},
-      {'sClass': 'expandable'},
-      {'sClass': 'expandable'},
-      {'sClass': 'expandable'}
-    ],
-    'fnDrawCallback': function () {
-      jQ('#testcases_table tbody tr td:nth-child(2)').shiftcheckbox({
-        checkboxSelector: ':checkbox',
-        selectAll: '#testcases_table .js-select-all'
-      });
-
-      jQ('#testcases_table :checkbox').on('change', function () {
-        let disable = jQ('#testcases_table tbody :checkbox:checked').length === 0;
-        jQ('#case_list_printable').prop('disabled', disable);
-        jQ('#clone_cases').prop('disabled', disable);
-        jQ('#export_selected_cases').prop('disabled', disable);
-      });
-    }
-  });
-
-  if (jQ('#id_checkbox_all_case')[0]) {
-    jQ('#id_checkbox_all_case').on('click', function () {
-      jQ('#case_list_printable').prop('disabled', !this.checked);
-    });
-  }
-
-  let listParams = Nitrate.TestCases.List.Param;
-  jQ('#case_list_printable').on('click', function () {
-    postToURL(listParams.case_printable, Nitrate.Utils.formSerialize(this.form));
-  });
-  jQ('#export_selected_cases').on('click', function () {
-    postToURL(listParams.case_export, Nitrate.Utils.formSerialize(this.form));
-  });
+Nitrate.TestCases.AdvancedSearch.on_load = function () {
+  Nitrate.TestCases.Search.initializeSearchResult('/advance-search/');
 };
 
 Nitrate.TestCases.Details.on_load = function () {
@@ -402,8 +382,10 @@ Nitrate.TestCases.Clone.on_load = function () {
     let container = jQ('#id_plan_container');
     container.show();
 
+    let thisform = e.target;
     sendHTMLRequest({
-      url: '/plans/',
+      url: thisform.action,
+      method: thisform.method,
       data: jQ(this).serialize(),
       container: container,
     });
@@ -457,7 +439,8 @@ function toggleSimpleCaseRunPane(options) {
  * A function bound to AJAX request success event to add or remove issue to and from a case. It
  * displays the issues table returned from the backend, and bind necessary event handlers, count and
  * display current number of issues added to case already.
- * @param {object} data
+ *
+ * @param {object} data - an object containing the data to display.
  * @param {string} data.html - a piece of HTML containing the issues table.
  */
 function issueOperationSuccessCallback(data) {
@@ -490,10 +473,9 @@ function addCaseIssue(form) {
     return false;
   }
 
-  getRequest({
+  postRequest({
     url: form.action,
     data: {
-      handle: 'add',
       issue_key: issueKey,
       tracker: parseInt(selectedIssueTrackerOption.val())
     },
@@ -507,9 +489,9 @@ function removeCaseIssue(issueKey, caseId, caseRunId) {
     message: 'Are you sure to remove issue ' + issueKey + '?',
     title: 'Manage Issues',
     yesFunc: function () {
-      getRequest({
-        url: '/case/' + caseId + '/issue/',
-        data: {handle: 'remove', issue_key: issueKey, case_run: caseRunId},
+      postRequest({
+        url: '/case/' + caseId + '/issues/delete/',
+        data: {issue_key: issueKey, case_run: caseRunId},
         success: issueOperationSuccessCallback,
         forbiddenMessage: 'You are not allowed to remove issue from case.',
       });
@@ -522,7 +504,8 @@ function removeCaseIssue(issueKey, caseId, caseRunId) {
 /**
  * Handle triggered by click event of Remove button to remove a plan from a
  * case' plans table. This is bound to specific element in the template directly.
- * @param {number} caseId
+ *
+ * @param {number} caseId - the case id.
  * @param {HTMLButtonElement} button - the element this handler is bound to.
  */
 function removePlanFromPlansTableHandler(caseId, button) {
@@ -544,8 +527,9 @@ function removePlanFromPlansTableHandler(caseId, button) {
 /**
  * Handler triggered by the form submit event to add plans to the case. This is
  * called in form submit event directly in the template.
- * @param {number} caseId
- * @param {HTMLFormElement} form
+ *
+ * @param {number} caseId - the case id.
+ * @param {HTMLFormElement} form - the form element containing elements having plan ids.
  */
 function addCaseToPlansHandler(caseId, form) {
   let planIds = form.elements['pk__in'].value.trim();
@@ -589,12 +573,15 @@ function addCaseIssueViaEnterKey(element, e) {
 
 /**
  * Toggle case runs by plan
- * @param {Object} params
- * @param {string|HTMLElement} params.c_container
- * @param {string|HTMLElement} params.container
- * @param {number} params.case_id
- * @param {number} params.case_run_plan_id
- * @param callback
+ *
+ * @param {object} params - an object containing information to toggle case runs.
+ * @param {string|HTMLElement} params.c_container - the container containing all the content
+ *                                                  including case runs.
+ * @param {string|HTMLElement} params.container - the container containing case runs.
+ * @param {number} params.case_id - the case id.
+ * @param {number} params.case_run_plan_id - the plan id.
+ * @param {Function} [callback] - a callback function passed to argument `callbackAfterFillIn` of
+ *                                function `sendHTMLRequest`.
  */
 function toggleCaseRunsByPlan(params, callback) {
   let contentContainer = typeof params.c_container === 'string' ?
