@@ -1,4 +1,5 @@
 Nitrate.TestRuns = {};
+Nitrate.TestRuns.Search = {};
 Nitrate.TestRuns.List = {};
 Nitrate.TestRuns.Details = {};
 Nitrate.TestRuns.New = {};
@@ -59,33 +60,9 @@ function toggleTestCaseContents(
   });
 }
 
-/**
- * Update run status.
- * @param contentType
- * @param objectPk
- * @param field
- * @param value
- * @param valueType
- * @param {function} callback - a function will be called when AJAX request succeeds. This function
- *                              accepts only one argument of the parsed JSON data returned from
- *                              server side.
- */
-function updateRunStatus(contentType, objectPk, field, value, valueType, callback) {
-  postRequest({
-    url: '/ajax/update/case-run-status',
-    success: callback,
-    data: {
-      content_type: contentType,
-      object_pk: Array.isArray(objectPk) ? objectPk.join(',') : objectPk,
-      field: field,
-      value: value,
-      value_type: valueType || 'str'
-    }
-  });
-}
-
 /**
  * Remove a case from test run new page.
+ *
  * @param {string} item - the HTML id of a container element containing the case to be removed.
  * @param {number} caseEstimatedTime - the case' estimated time.
  */
@@ -118,20 +95,66 @@ function cloneRunsClickHandler() {
   postToURL(jQ(this).data('param'), Nitrate.Utils.formSerialize(this.form), 'get');
 }
 
+
+/**
+ * Initialize the test runs search result table and associated action buttons.
+ *
+ * @param {string} searchEndpoint - the endpoint to search test runs.
+ */
+Nitrate.TestRuns.Search.initializeSearchResult = function (searchEndpoint) {
+  let runsSearchResultTableSettings = Object.assign({}, Nitrate.DataTable.commonSettings, {
+    aaSorting: [[1, 'desc']],
+    sAjaxSource: searchEndpoint + window.location.search,
+
+    iDeferLoading: Nitrate.TestRuns.Search.numberOfRuns,
+
+    aoColumns: [
+      {'bSortable': false},       // Select checker
+      {'sType': 'numeric'},       // ID
+      {'sType': 'html'},          // Summary
+      {'sType': 'html'},          // Manager
+      {'sType': 'html'},          // Default Tester
+      null,                       // Product
+      null,                       // Product Version
+      null,                       // Environment
+      {'sType': 'numeric'},       // Cases
+      {'sType': 'html'},          // Status
+      {'bSortable': false}        // Completed progress
+    ],
+
+    oLanguage: {
+      sEmptyTable: 'No run was found.'
+    },
+
+    fnInitComplete: function () {
+      jQ('.js-clone-testruns').on('click', cloneRunsClickHandler);
+    },
+
+    fnDrawCallback: function () {
+      jQ('#testruns_table tbody tr td:nth-child(1)').shiftcheckbox({
+        checkboxSelector: ':checkbox',
+        selectAll: '#testruns_table .js-select-all'
+      });
+
+      jQ('#testruns_table :checkbox').on('change', function () {
+        jQ('.js-clone-testruns').prop(
+          'disabled', jQ('#testruns_table tbody :checkbox:checked').length === 0
+        );
+      });
+    },
+
+    fnInfoCallback: function (oSettings, iStart, iEnd, iMax, iTotal, sPre) {
+      return 'Showing ' + (iEnd - iStart + 1) + ' of ' + iTotal + ' runs';
+    }
+
+  });
+
+  jQ('#testruns_table').dataTable(runsSearchResultTableSettings);
+};
+
 Nitrate.TestRuns.AdvancedSearch.on_load = function () {
-  jQ('#testruns_table tbody tr td:nth-child(1)').shiftcheckbox({
-    checkboxSelector: ':checkbox',
-    selectAll: '#testruns_table .js-select-all',
-  });
-
-  jQ('#testruns_table :checkbox').on('change', function () {
-    jQ('.js-clone-testruns').prop(
-      'disabled', jQ('#testruns_table tbody :checkbox:checked').length === 0
-    );
-  });
-
-  jQ('.js-clone-testruns').on('click', cloneRunsClickHandler);
-}
+  Nitrate.TestRuns.Search.initializeSearchResult('/advance-search/');
+};
 
 Nitrate.TestRuns.List.on_load = function () {
   registerProductAssociatedObjectUpdaters(
@@ -172,39 +195,7 @@ Nitrate.TestRuns.List.on_load = function () {
     });
   }
 
-  jQ('#testruns_table').dataTable({
-    'iDisplayLength': 20,
-    'sPaginationType': 'full_numbers',
-    'bFilter': false,
-    'bLengthChange': false,
-    'aaSorting': [[ 1, 'desc' ]],
-    'bProcessing': true,
-    'bServerSide': true,
-    'sAjaxSource': '/runs/ajax/' + this.window.location.search,
-    'aoColumns': [
-      {'bSortable': false},
-      {'sType': 'numeric'},
-      {'sType': 'html'},
-      {'sType': 'html'},
-      {'sType': 'html'},
-      {'bVisible': false},
-      null,
-      null,
-      null,
-      {'sType': 'numeric', 'bSortable': false},
-      null,
-      {'bSortable': false}
-    ],
-    'oLanguage': {'sEmptyTable': 'No run was found.'},
-    'fnDrawCallback': function () {
-      jQ('#testruns_table tbody tr td:nth-child(1)').shiftcheckbox({
-        checkboxSelector: ':checkbox',
-        selectAll: '#testruns_table .js-select-all'
-      });
-    }
-  });
-
-  jQ('.js-clone-testruns').on('click', cloneRunsClickHandler);
+  Nitrate.TestRuns.Search.initializeSearchResult('/runs/');
 };
 
 
@@ -610,7 +601,9 @@ Nitrate.TestRuns.Details.on_load = function () {
     window.location.href = params[0] + '?from_plan=' + params[1];
   });
   jQ('#btn_clone').on('click', function () {
-    postToURL(jQ(this).data('param'), getSelectedCaseRunIDs());
+    postToURL(jQ(this).data('param'), {
+      case_run: getSelectedCaseRunIDs()
+    });
   });
   jQ('#btn_delete').on('click', function () {
     window.location.href = jQ(this).data('param');
@@ -638,7 +631,9 @@ Nitrate.TestRuns.Details.on_load = function () {
     delCaseRun();
   });
   jQ('.js-update-case').on('click', function () {
-    postToURL(jQ(this).data('param'), getSelectedCaseRunIDs());
+    postToURL(jQ(this).data('param'), {
+      case_run: getSelectedCaseRunIDs()
+    });
   });
   jQ('.js-change-assignee').on('click', function () {
     changeCaseRunAssignee();
@@ -840,7 +835,11 @@ Nitrate.TestRuns.AssignCase.on_load = function () {
 /**
  * A callback called after a comment is added to a case run or a case run status is updated with a
  * short comment.
- * @callback
+ *
+ * @param {jQuery} caseRunRow - the container representing the case run table row.
+ * @param {jQuery} expandedCaseRunDetailRow - the container representing the expanded table row
+ *                                            containing case run details.
+ * @param {string} caseRunStatusId - case run status ID.
  */
 function updateCaseRunDetailAfterCommentIsAdded(caseRunRow, expandedCaseRunDetailRow, caseRunStatusId) {
   // Update the contents
@@ -874,7 +873,7 @@ function updateCaseRunDetailAfterCommentIsAdded(caseRunRow, expandedCaseRunDetai
   if (jQ('#id_check_box_auto_blinddown').prop('checked') && caseRunStatusId !== '') {
     let nextTitle = expandedCaseRunDetailRow.next();
     if (!nextTitle.length) {
-      return false;
+      return;
     }
     if (nextTitle.next().is(':hidden')) {
       nextTitle.find('.expandable').trigger('click');
@@ -886,8 +885,8 @@ function updateCaseRunDetailAfterCommentIsAdded(caseRunRow, expandedCaseRunDetai
 
 /**
  * A function registered to the form submit event, from where to add comment to or change status for a case run.
- * @callback
- * @param e
+ *
+ * @param {Event} e - the DOM event.
  */
 function updateCaseRunStatus(e) {
   e.stopPropagation();
@@ -918,11 +917,17 @@ function updateCaseRunStatus(e) {
     // Reset the content to loading
     caseRunDetailCell.html(constructAjaxLoading('id_loading_' + formData.case_id));
 
-    updateRunStatus(
-      formData.content_type, formData.object_pk, formData.field, caseRunStatusId, 'int',
-      function () {
+    updateObject({
+      url: '/ajax/update/case-run-status',
+      contentType: formData.content_type,
+      objectPk: formData.object_pk,
+      field: formData.field,
+      value: caseRunStatusId,
+      valueType: 'int',
+      callback: function () {
         updateCaseRunDetailAfterCommentIsAdded(caseRunRow, expandedCaseRunDetailRow, caseRunStatusId);
-      });
+      }
+    });
   }
 }
 
@@ -951,7 +956,13 @@ function changeCaseRunOrder(runId, caseRunId, sortKey) {
     return false;
   }
 
-  updateObject('testruns.testcaserun', caseRunId, 'sortkey', nsk, 'int');
+  updateObject({
+    contentType: 'testruns.testcaserun',
+    objectPk: caseRunId,
+    field: 'sortkey',
+    value: nsk,
+    valueType: 'int'
+  });
 }
 
 function taggleSortCaseRun(event) {
@@ -982,7 +993,9 @@ function taggleSortCaseRun(event) {
     jQ('#id_table_cases').tableDnD();
   } else {
     jQ('#id_table_cases input[type=checkbox]').prop({'checked': true, 'disabled': false});
-    postToURL('ordercaserun/', getSelectedCaseRunIDs());
+    postToURL('ordercaserun/', {
+      case_run: getSelectedCaseRunIDs()
+    });
   }
 }
 
@@ -1029,17 +1042,16 @@ function removeIssueFromCaseRuns(removeIssueInfo, reloadInfo) {
 }
 
 function delCaseRun() {
-  let caseruns = getSelectedCaseRunIDs();
-  let numCaseRuns = caseruns.case_run.length;
+  let caseRunIDs = getSelectedCaseRunIDs();
 
-  if (numCaseRuns === 0) {
+  if (caseRunIDs.length === 0) {
     return;
   }
 
   confirmDialog({
-    message: 'You are about to delete ' + numCaseRuns + ' case run(s). Are you sure?',
+    message: 'You are about to delete ' + caseRunIDs.length + ' case run(s). Are you sure?',
     yesFunc: function () {
-      postToURL('removecaserun/', caseruns);
+      postToURL('removecaserun/', {'case_run': caseRunIDs});
     }
   });
 }
@@ -1096,13 +1108,12 @@ function submitValue(runId, value, hidebox, selectField, submitid) {
     return false;
   }
 
-  getRequest({
-    url: '/runs/env_value/',
+  postRequest({
+    url: '/runs/env_value/change/',
     data: {
-      a: 'change',
-      old_env_value_id: oldValue,
-      new_env_value_id: selectField.value,
-      run_id: runId
+      old_env_value: oldValue,
+      new_env_value: selectField.value,
+      runs: runId
     },
     success: function () {
       jQ('#' + hidebox).html(newValue).show();
@@ -1122,15 +1133,10 @@ function removeProperty(runId, element) {
       let emptySelf = jQ(element).closest('li');
       let envValueId = jQ('input[type=hidden][name=current_run_env]', parent).get(0).value;
 
-      getRequest({
-        url: '/runs/env_value/',
-        data: {
-          a: 'remove',
-          info_type: 'env_values',
-          env_value_id: envValueId,
-          run_id: runId
-        },
-        errorMessage: 'Edit value failed',
+      postRequest({
+        url: '/runs/env_value/delete/',
+        data: {env_value: envValueId, runs: runId},
+        errorMessage: 'Deleting value failed',
         success: function () { emptySelf.remove(); },
       });
     }
@@ -1138,14 +1144,9 @@ function removeProperty(runId, element) {
 }
 
 function addPropertyToEnv(runId, envValueId) {
-  getRequest({
-    url: '/runs/env_value/',
-    data: {
-      a: 'add',
-      info_type: 'env_values',
-      env_value_id: envValueId,
-      run_id: runId
-    },
+  postRequest({
+    url: '/runs/env_value/add/',
+    data: {env_value: envValueId, runs: runId},
     success: function (data) {
       jQ('#dialog').hide();
       jQ('#env_area').html(data.fragment);
@@ -1235,7 +1236,7 @@ function removeRunCC(runId, user, container) {
 }
 
 function changeCaseRunAssignee() {
-  let selectedCaseRunIDs = getSelectedCaseRunIDs().case_run;
+  let selectedCaseRunIDs = getSelectedCaseRunIDs();
   if (!selectedCaseRunIDs.length) {
     showModal(defaultMessages.alert.no_case_selected, 'Missing something?');
     return false;
@@ -1264,22 +1265,29 @@ function changeCaseRunAssignee() {
         return false;
       }
 
-      updateObject('testruns.testcaserun', selectedCaseRunIDs, 'assignee', data[0].pk, 'str');
+      updateObject({
+        contentType: 'testruns.testcaserun',
+        objectPk: selectedCaseRunIDs,
+        field: 'assignee',
+        value: data[0].pk,
+        valueType: 'int'
+      });
     },
   });
 }
 
 /**
  * Retrieve and return selected case run IDs from the container table whose id is id_table_cases.
- * @returns {{case_run: *}}
+ *
+ * @returns {string[]} the selected test case IDs.
  */
 function getSelectedCaseRunIDs() {
-  return {
-    case_run:
-      jQ('#id_table_cases input[name="case_run"]:checked')
-        .map(function () {return this.value;})
-        .get()
+  let result = []
+    , checkedCaseRuns = jQ('#id_table_cases input[name="case_run"]:checked');
+  for (let i = 0; i < checkedCaseRuns.length; i++) {
+    result.push(checkedCaseRuns[i].value);
   }
+  return result;
 }
 
 function showCaseRunsWithSelectedStatus(form, statusId) {
@@ -1321,9 +1329,9 @@ function insertCasesIntoTestRun() {
  * Click event handler for A .js-add-issues
  */
 function addIssueToBatchCaseRunsHandler() {
-  let caseRunIds =
-    getSelectedCaseRunIDs()
-      .case_run.map(function (item) {return parseInt(item);});
+  let caseRunIds = getSelectedCaseRunIDs().map(function (item) {
+    return parseInt(item);
+  });
   if (caseRunIds.length === 0) {
     showModal(defaultMessages.alert.no_case_selected, 'Missing something?');
   } else {
@@ -1340,9 +1348,9 @@ function addIssueToBatchCaseRunsHandler() {
  * Click event handler for A .js-remove-issues
  */
 function removeIssueFromBatchCaseRunsHandler() {
-  let caseRunIds =
-    getSelectedCaseRunIDs()
-      .case_run.map(function (item) {return parseInt(item);});
+  let caseRunIds = getSelectedCaseRunIDs().map(function (item) {
+    return parseInt(item);
+  });
 
   if (caseRunIds.length === 0) {
     showModal(defaultMessages.alert.no_case_selected, 'Missing something?');
@@ -1376,7 +1384,7 @@ function removeIssueFromBatchCaseRunsHandler() {
 }
 
 function showCommentForm() {
-  let caseRunIds = getSelectedCaseRunIDs().case_run;
+  let caseRunIds = getSelectedCaseRunIDs();
 
   if (caseRunIds.length === 0) {
     showModal(defaultMessages.alert.no_case_selected, 'Missing something?');
@@ -1429,7 +1437,7 @@ jQ(document).ready(function (){
     if (option === '') {
       return false;
     }
-    let objectPks = getSelectedCaseRunIDs().case_run;
+    let objectPks = getSelectedCaseRunIDs();
     if (!objectPks.length) {
       showModal(defaultMessages.alert.no_case_selected, 'Missing something?');
       return false;
@@ -1438,7 +1446,13 @@ jQ(document).ready(function (){
     confirmDialog({
       message: defaultMessages.confirm.change_case_status,
       yesFunc: function () {
-        updateObject('testruns.testcaserun', objectPks, 'case_run_status', option, 'int');
+        updateObject({
+          contentType: 'testruns.testcaserun',
+          objectPk: objectPks,
+          field: 'case_run_status',
+          value: option,
+          valueType: 'int'
+        });
       }
     });
   });
@@ -1451,7 +1465,7 @@ function getAddLinkDialog() {
 /**
  * Do AJAX request to backend to remove a link
  *
- * @param sender:
+ * @param {HTMLElement} sender - the HTML element triggering the event to remove a link.
  * @param {number} linkId - the ID of an arbitrary link.
  */
 function removeLink(sender, linkId) {
@@ -1465,9 +1479,9 @@ function removeLink(sender, linkId) {
 /**
  * Add link to case run
  *
- * @param sender - the Add link button, which is pressed to fire this event.
- * @param {number} caseId
- * @param {number} caseRunId
+ * @param {HTMLElement} sender - the Add link button, which is pressed to fire this event.
+ * @param {number} caseId - the test case ID.
+ * @param {number} caseRunId - the test case run ID.
  */
 function addLinkToCaseRun(sender, caseId, caseRunId) {
   let dialog = getAddLinkDialog();
@@ -1487,7 +1501,7 @@ function addLinkToCaseRun(sender, caseId, caseRunId) {
  * Initialize dialog for getting information about new link, which is attached
  * to an arbitrary instance of TestCaseRun
  *
- * @param linkTarget - string, the name of Model to whose instance new link will be linked.
+ * @param {string} linkTarget - the name of Model to whose instance new link will be linked.
  */
 function initializeAddLinkDialog(linkTarget) {
   let dialog = getAddLinkDialog();
@@ -1553,13 +1567,14 @@ function initializeAddLinkDialog(linkTarget) {
 /**
  * Toggle TestCaseRun panel to edit a case run in run page.
  *
- * @param {Object} options
- * @param {HTMLElement} options.caserunRowContainer
- * @param {HTMLElement} options.expandPaneContainer
- * @param {number} options.caseId
- * @param {number} options.caserunId
- * @param {number} options.caseTextVersion
- * @param {function} options.callback
+ * @param {object} options - options for toggling the test case run pane.
+ * @param {HTMLElement} options.caserunRowContainer - the container element containing the test case run row.
+ * @param {HTMLElement} options.expandPaneContainer - the container element to be expanded for the associated test case
+ *                                                    run.
+ * @param {number} options.caseId - the associated test case ID.
+ * @param {number} options.caserunId - the test case run ID.
+ * @param {number} options.caseTextVersion - the test case' text version.
+ * @param {Function} options.callback - a function called after the content is filled in the expanded container.
  */
 function toggleTestCaseRunPane(options) {
   let container = options.expandPaneContainer;
