@@ -4,5 +4,5 @@ from django.urls import path
 from . import views
 
 urlpatterns = [
-    path('', views.advance_search, name='advance_search'),
+    path('', views.advance_search, name='advanced-search'),
 ]
