@@ -5,7 +5,6 @@ import itertools
 import json
 import logging
 
-from http import HTTPStatus
 from operator import itemgetter, attrgetter
 from typing import Dict, List, Optional
 
@@ -19,7 +18,6 @@ from django.core.exceptions import ValidationError
 from django.db.models import Count
 from django.http import HttpResponseBadRequest, JsonResponse
 from django.http import HttpResponseRedirect, HttpResponse, Http404
-from django import forms as djforms
 from django.shortcuts import get_object_or_404, render
 from django.template.loader import get_template
 from django.template.loader import render_to_string
@@ -31,21 +29,21 @@ from django.views.generic.edit import FormView
 
 from tcms.core.db import SQLExecution
 from tcms.core.raw_sql import RawSQL
-from tcms.core.responses import JsonResponseBadRequest, JsonResponseForbidden
+from tcms.core.responses import JsonResponseBadRequest
 from tcms.core.utils import DataTableResult
-from tcms.core.utils import form_error_messags_to_list
+from tcms.core.utils import form_error_messages_to_list
 from tcms.core.views import prompt
 from tcms.issuetracker.models import IssueTracker
 from tcms.logs.models import TCMSLogModel
 from tcms.management.models import Priority, TestTag
-from tcms.search.order import order_case_queryset
+from tcms.search.order import apply_order
 from tcms.search.views import remove_from_request_path
 from tcms.testcases.data import get_exported_cases_and_related_data
 from tcms.testcases.fields import CC_LIST_DEFAULT_DELIMITER
 from tcms.testcases.forms import (
     CaseAutomatedForm, NewCaseForm, SearchCaseForm, CaseFilterForm,
     EditCaseForm, CaseNotifyForm, CloneCaseForm, CaseIssueForm, CaseTagForm,
-    CaseComponentForm, CasePlansForm
+    CaseComponentForm, CasePlansForm, CaseRemoveIssueForm
 )
 from tcms.testcases import actions
 from tcms.testcases import data
@@ -156,7 +154,7 @@ class ChangeCaseAutomatedPropertyView(PermissionRequiredMixin, FormView):
 
     def form_invalid(self, form):
         return JsonResponseBadRequest({
-            'messages': form_error_messags_to_list(form)
+            'messages': form_error_messages_to_list(form)
         }, safe=True)
 
 
@@ -346,7 +344,7 @@ def sort_queried_testcases(request, testcases):
     """
     order_by = request.POST.get('order_by', 'create_date')
     asc = bool(request.POST.get('asc', None))
-    tcs = order_case_queryset(testcases, order_by, asc)
+    tcs = apply_order(testcases, order_by, asc)
     # default sorted by sortkey
     tcs = tcs.order_by('testcaseplan__sortkey')
     # Resort the order
@@ -509,125 +507,49 @@ def all(request, template_name="case/all.html"):
 
 
 @require_GET
-def search(request, template_name='case/all.html'):
-    """
-    generate the function of searching cases with search criteria
-    """
-    # This is ugly, but it's not a big deal at this moment because I'm going go
-    # rewrite the search functionality as well as for runs and plans.
-    search_criterias = len(list(request.GET.items())) > 0
-
-    search_form = SearchCaseForm(request.GET)
-    if request.GET.get('product'):
-        search_form.populate(product_id=request.GET['product'])
-    else:
-        search_form.populate()
-
-    context_data = {
-        'module': MODULE_NAME,
-        'search_form': search_form,
-        'search_criterias': search_criterias,
-    }
-    return render(request, template_name, context=context_data)
-
-
-@require_GET
-def ajax_search(request, template_name='case/common/json_cases.txt'):
+def search_cases(request):
     """Generate the case list in search case and case zone in plan
     """
-    SearchForm = SearchCaseForm
+    search_form = SearchCaseForm(request.GET)
 
-    tp = plan_from_request_or_none(request)
-
-    action = request.GET.get('a')
-
-    # Initial the form and template
-    if action in ('search', 'sort'):
-        search_form = SearchForm(request.GET)
+    if search_form.is_valid():
+        cases = (
+            TestCase
+            .list(search_form.cleaned_data)
+            .select_related('author', 'default_tester', 'case_status',
+                            'priority', 'category')
+            .only('pk', 'summary', 'author__username', 'default_tester__username',
+                  'is_automated', 'case_status__name', 'category__name',
+                  'priority__value', 'is_automated_proposed',  # 'default_tester__id',
+                  'create_date')
+            .order_by('-create_date')
+        )
     else:
-        # Hacking for case plan
-        confirmed_status_name = 'CONFIRMED'
-        # 'c' is meaning component
-        template_type = request.GET.get('template_type')
-        if template_type == 'case':
-            d_status = TestCaseStatus.objects.filter(name=confirmed_status_name)
-        elif template_type == 'review_case':
-            d_status = TestCaseStatus.objects.exclude(name=confirmed_status_name)
-        else:
-            d_status = TestCaseStatus.objects.all()
-
-        d_status_ids = d_status.values_list('pk', flat=True)
-
-        search_form = SearchForm(initial={'case_status': d_status_ids})
-
-    # Populate the form
-    if request.GET.get('product'):
-        search_form.populate(product_id=request.GET['product'])
-    elif tp and tp.product_id:
-        search_form.populate(product_id=tp.product_id)
-    else:
-        search_form.populate()
-
-    # Query the database when search
-    if action in ('search', 'sort') and search_form.is_valid():
-        tcs = TestCase.list(search_form.cleaned_data)
-    elif action == 'initial':
-        tcs = TestCase.objects.filter(case_status__in=d_status)
-    else:
-        tcs = TestCase.objects.none()
-
-    # Search the relationship
-    if tp:
-        tcs = tcs.filter(plan=tp)
-
-    tcs = tcs.select_related(
-        'author',
-        'default_tester',
-        'case_status',
-        'priority',
-        'category'
-    ).only(
-        'case_id',
-        'summary',
-        'create_date',
-        'is_automated',
-        'is_automated_proposed',
-        'case_status__name',
-        'category__name',
-        'priority__value',
-        'author__username',
-        'default_tester__id',
-        'default_tester__username'
-    )
-    tcs = tcs.extra(select={'num_issue': RawSQL.num_case_issues, })
+        cases = TestCase.objects.none()
 
     # columnIndexNameMap is required for correct sorting behavior, 5 should be
     # product, but we use run.build.product
     column_names = [
-        '',
-        '',
-        'case_id',
-        'summary',
-        'author__username',
-        'default_tester__username',
-        'is_automated',
-        'case_status__name',
-        'category__name',
-        'priority__value',
-        'create_date',
+        '', '', 'pk', 'summary', 'author__username',
+        'default_tester__username', 'is_automated', 'case_status__name',
+        'category__name', 'priority__value', 'create_date',
     ]
-    return ajax_response(request, tcs, column_names, template_name)
 
-
-def ajax_response(request, queryset, column_names, template_name):
-    """json template for the ajax request for searching"""
-    dt = DataTableResult(request.GET, queryset, column_names)
-
-    # prepare the JSON with the response, consider using :
-    # from django.template.defaultfilters import escapejs
-    t = get_template(template_name)
-    resp_data = t.render(dt.get_response_data(), request)
-    return JsonResponse(json.loads(resp_data))
+    if 'sEcho' in request.GET:
+        dt = DataTableResult(request.GET, cases, column_names,
+                             default_order_key='-pk')
+        resp_data = (get_template('case/common/json_cases.txt')
+                     .render(dt.get_response_data(), request))
+        return JsonResponse(json.loads(resp_data))
+    else:
+        context_data = {
+            'module': 'testruns',
+            'sub_module': 'cases',
+            'object_list': cases[0:20],
+            'search_form': search_form,
+            'total_count': cases.count(),
+        }
+        return render(request, 'case/all.html', context=context_data)
 
 
 class SimpleTestCaseView(TemplateView, data.TestCaseViewDataMixin):
@@ -1394,7 +1316,7 @@ class AddComponentView(PermissionRequiredMixin, View):
         form.populate(product_id=request.POST['product'])
         if not form.is_valid():
             return JsonResponseBadRequest({
-                'message': form_error_messags_to_list(form),
+                'message': form_error_messages_to_list(form),
             }, safe=True)
 
         case_ids = [int(case_id) for case_id in request.POST.getlist('case')]
@@ -1438,7 +1360,7 @@ class RemoveComponentView(PermissionRequiredMixin, View):
 
         if not form.is_valid():
             return JsonResponseBadRequest({
-                'message': form_error_messags_to_list(form),
+                'message': form_error_messages_to_list(form),
             })
 
         errors = []
@@ -1524,117 +1446,85 @@ def get_log(request, case_id, template_name="management/get_log.html"):
     return render(template_name, context=context_data)
 
 
-@permission_required('issuetracker.change_issue')
-def manage_case_issues(request, case_id, template_name='case/get_issues.html'):
-    """Process the bugs for cases"""
+class CasesIssueActionBaseView(PermissionRequiredMixin, FormView):
+    """Base view class of actions on cases' issues"""
 
-    class CaseIssueActions:
-        __all__ = ['get_form', 'render', 'add', 'remove']
+    raise_exception = True
 
-        def __init__(self, request, case, template_name):
-            self.request = request
-            self.case = case
-            self.template_name = template_name
+    def get_form(self, form_class=None):
+        post_data = self.request.POST.copy()
+        post_data['case'] = self.kwargs['case_id']
+        return self.form_class(post_data)
 
-        def render_form(self):
-            form = CaseIssueForm(initial={'case': self.case})
-            if request.GET.get('type') == 'table':
-                return HttpResponse(form.as_table())
+    def do(self, form):
+        """Do the real action on the issues
 
-            return HttpResponse(form.as_p())
+        :param form: the form provided by FormView.
+        :type form: :class:`django.forms.Form`
+        :return: a Django response object if something is wrong, otherwise None is returned.
+        :rtype: :class:`JsonResponseBadRequest` or None
+        """
+        raise NotImplementedError('Must be implemented in subclass.')
 
-        def render(self, response=None):
-            context_data = {
-                'test_case': self.case,
-                'issue_trackers': IssueTracker.get_by_case(self.case),
-                'response': response
-            }
-            return render_to_string(template_name, context_data, request)
+    def form_valid(self, form):
+        response = self.do(form)
 
-        def add(self):
-            # FIXME: It's may use ModelForm.save() method here.
-            #        Maybe in future.
-            if not self.request.user.has_perm('issuetracker.add_issue'):
-                return JsonResponse({'messages': 'Permission denied.'},
-                                    status=HTTPStatus.FORBIDDEN)
+        if response is not None:
+            return response
 
-            request_data = request.GET.copy()
-            request_data.update({'case': self.case.pk})
-            form = CaseIssueForm(request_data)
-            if not form.is_valid():
-                return JsonResponseBadRequest({
-                    'messages': form_error_messags_to_list(form)
-                })
+        case = form.cleaned_data['case']
+        context = {
+            'test_case': case,
+            'issue_trackers': IssueTracker.get_by_case(case),
+        }
+        return JsonResponse({
+            'html': render_to_string('case/get_issues.html', context, self.request)
+        })
 
-            try:
-                self.case.add_issue(
-                    issue_key=form.cleaned_data['issue_key'],
-                    issue_tracker=form.cleaned_data['tracker'],
-                    summary=form.cleaned_data['summary'],
-                    description=form.cleaned_data['description'],
-                )
-            except ValidationError as e:
-                return JsonResponseBadRequest({'messages': e.messages})
-            except Exception as e:
-                msg = 'Failed to add issue {} to case {}. Error reported: {}'.format(
-                    form.cleaned_data['issue_key'], self.case.pk, str(e))
-                logger.exception(msg)
-                return JsonResponseBadRequest({'messages': msg})
+    def form_invalid(self, form):
+        return JsonResponseBadRequest({
+            'message': form_error_messages_to_list(form)
+        })
 
-            return JsonResponse({'html': self.render()})
 
-        def remove(self):
-            if not self.request.user.has_perm('issuetracker.delete_issue'):
-                return JsonResponseForbidden({'messages': 'Permission denied.'})
+class AddIssueToCases(CasesIssueActionBaseView):
+    """Add an issue to a test cases"""
 
-            class CaseRemoveIssueForm(djforms.Form):
-                handle = djforms.RegexField(r'^remove$')
-                issue_key = djforms.CharField(
-                    min_length=1, max_length=50,
-                    error_messages={
-                        'required': 'Missing issue key to delete.'
-                    }
-                )
-                case_run = djforms.ModelChoiceField(
-                    required=False,
-                    queryset=TestCaseRun.objects.all(),
-                    error_messages={
-                        'invalid_choice': 'Test case run does not exists.'
-                    }
-                )
+    form_class = CaseIssueForm
+    permission_required = 'issuetracker.add_issue'
 
-            form = CaseRemoveIssueForm(request.GET)
-            if form.is_valid():
-                try:
-                    self.case.remove_issue(form.cleaned_data['issue_key'],
-                                           form.cleaned_data['case_run'])
-                except (TypeError, ValueError) as e:
-                    return JsonResponseBadRequest({'messages': str(e)})
-                else:
-                    return JsonResponse({'html': self.render()})
-            else:
-                return JsonResponseBadRequest({
-                    'messages': form_error_messags_to_list(form)
-                })
+    def do(self, form):
+        case = form.cleaned_data['case']
+        try:
+            case.add_issue(
+                issue_key=form.cleaned_data['issue_key'],
+                issue_tracker=form.cleaned_data['tracker'],
+                summary=form.cleaned_data['summary'],
+                description=form.cleaned_data['description'],
+            )
+        except ValidationError as e:
+            return JsonResponseBadRequest({'message': e.messages})
+        except Exception as e:
+            msg = 'Failed to add issue {} to case {}. Error reported: {}'.format(
+                form.cleaned_data['issue_key'], case.pk, str(e)
+            )
+            logger.exception(msg)
+            return JsonResponseBadRequest({'message': msg})
 
-    # FIXME: Rewrite these codes for Ajax.Request
-    try:
-        tc = get_object_or_404(TestCase, case_id=case_id)
-    except Http404:
-        return JsonResponse(
-            {'messages': [f'Case {case_id} does not exist.']},
-            status=HTTPStatus.NOT_FOUND)
 
-    actions = CaseIssueActions(request=request,
-                               case=tc,
-                               template_name=template_name)
+class DeleteIssueFromCases(CasesIssueActionBaseView):
+    """Delete an issue from cases"""
 
-    if not request.GET.get('handle') in actions.__all__:
-        return JsonResponse({'messages': 'Unrecognizable actions'},
-                            status=HTTPStatus.BAD_REQUEST)
+    form_class = CaseRemoveIssueForm
+    permission_required = 'issuetracker.delete_issue'
 
-    func = getattr(actions, request.GET['handle'])
-    return func()
+    def do(self, form):
+        case = form.cleaned_data['case']
+        try:
+            case.remove_issue(form.cleaned_data['issue_key'],
+                              form.cleaned_data['case_run'])
+        except (TypeError, ValueError) as e:
+            return JsonResponseBadRequest({'message': str(e)})
 
 
 class CasePlansOperationView(PermissionRequiredMixin, View):
@@ -1648,7 +1538,7 @@ class CasePlansOperationView(PermissionRequiredMixin, View):
 
         if not form.is_valid():
             return JsonResponseBadRequest({
-                'message': form_error_messags_to_list(form)
+                'message': form_error_messages_to_list(form)
             })
 
         self.operate(form)
