@@ -14,8 +14,6 @@ from django.contrib.auth.decorators import permission_required
 from django.contrib.auth.mixins import PermissionRequiredMixin
 from django.core.exceptions import ObjectDoesNotExist
 from django.urls import reverse
-from django.db.models import Count
-from django.db.models import Q
 from django.http import Http404, HttpResponsePermanentRedirect
 from django.http import HttpResponse, HttpResponseRedirect
 from django.http import HttpResponseBadRequest, JsonResponse
@@ -25,6 +23,7 @@ from django.views.decorators.csrf import csrf_protect
 from django.views.decorators.http import require_GET
 from django.views.decorators.http import require_http_methods
 from django.views.generic import View
+from django.views.generic.base import TemplateView
 from tcms.core.responses import JsonResponseBadRequest
 from uuslug import slugify
 
@@ -35,8 +34,6 @@ from tcms.core.utils import checksum
 from tcms.core.utils import DataTableResult
 from tcms.core.views import prompt
 from tcms.management.models import TCMSEnvGroup, Component
-from tcms.search.views import remove_from_request_path
-from tcms.search.order import order_plan_queryset
 from tcms.testcases.data import get_exported_cases_and_related_data
 from tcms.testcases.forms import SearchCaseForm, QuickSearchCaseForm
 from tcms.testcases.models import TestCaseStatus
@@ -50,7 +47,6 @@ from tcms.testplans.forms import PlanComponentForm
 from tcms.testplans.forms import SearchPlanForm
 from tcms.testplans import sqls
 from tcms.testplans.models import TestPlan, TestPlanComponent
-from tcms.testplans.models import TestPlanEmailSettings
 from tcms.testruns.models import TestRun, TestCaseRun
 
 
@@ -129,8 +125,6 @@ class CreateNewPlanView(PermissionRequiredMixin, View):
             parent=form.cleaned_data['parent'],
         )
 
-        TestPlanEmailSettings.objects.create(plan=tp)
-
         tp.add_text(author=request.user, plan_text=form.cleaned_data['text'])
 
         # Add test plan environment groups
@@ -249,246 +243,121 @@ def delete(request, plan_id):
         return prompt.info(request, 'Nothing yet.')
 
 
-@require_GET
-def all(request, template_name='plan/all.html'):
-    """Display all testplans"""
-    # Define the default sub module
-    SUB_MODULE_NAME = 'plans'
-    # TODO: this function now only performs a forward feature, no queries
-    # need here. All of it will be removed in the future.
-    # If it's not a search the page will be blank
-    tps = TestPlan.objects.none()
-    query_result = False
-    order_by = request.GET.get('order_by', 'create_date')
-    asc = bool(request.GET.get('asc', None))
-    # if it's a search request the page will be fill
-    if list(request.GET.items()):
-        search_form = SearchPlanForm(request.GET)
-        if request.GET.get('product'):
-            search_form.populate(product_id=request.GET['product'])
-        else:
-            search_form.populate()
+class SimplePlansFilterView(TemplateView):
+    """Providing base plans filter functionaity"""
+
+    # Subclass should provide a concrete template to render the final content.
+    # Or, pass the template path to argument template_name of View.as_view()
+    template_name = None
+
+    def filter_plans(self):
+        search_form = SearchPlanForm(self.request.GET)
+        product_id = self.request.GET.get('product')
+        search_form.populate(int(product_id) if product_id else None)
+
+        plans = TestPlan.objects.none()
 
         if search_form.is_valid():
-            # Detemine the query is the user's plans and change the sub
-            # module value
-            author = request.GET.get('author')
-            if author and request.user.is_authenticated:
-                if author == request.user.username or author == request.user.email:
-                    SUB_MODULE_NAME = "my_plans"
+            # Determine the query is the user's plans and change the sub module value
+            author = self.request.GET.get('author__email__startswith')
+            req_user = self.request.user
+            if req_user.is_authenticated and author in (req_user.username, req_user.email):
+                self.SUB_MODULE_NAME = "my_plans"
 
-            query_result = True
-            # build a QuerySet:
-            tps = TestPlan.list(search_form.cleaned_data)
-            tps = tps.select_related('author', 'type', 'product')
+            plans = (
+                TestPlan
+                .list(search_form.cleaned_data)
+                .select_related('author', 'type', 'product')
 
-            # We want to get the number of cases and runs, without doing
-            # lots of per-test queries.
-            #
-            # Ideally we would get the case/run counts using m2m field tricks
-            # in the ORM
-            # Unfortunately, Django's select_related only works on ForeignKey
-            # relationships, not on ManyToManyField attributes
-            # See http://code.djangoproject.com/ticket/6432
+                # We want to get the number of cases and runs, without doing
+                # lots of per-test queries.
+                #
+                # Ideally we would get the case/run counts using m2m field tricks
+                # in the ORM
+                # Unfortunately, Django's select_related only works on ForeignKey
+                # relationships, not on ManyToManyField attributes
+                # See http://code.djangoproject.com/ticket/6432
 
-            # SQLAlchemy can handle this kind of thing in several ways.
-            # Unfortunately we're using Django
+                # SQLAlchemy can handle this kind of thing in several ways.
+                # Unfortunately we're using Django
 
-            # The cleanest way I can find to get it into one query is to
-            # use QuerySet.extra()
-            # See http://docs.djangoproject.com/en/dev/ref/models/querysets
-            tps = tps.extra(select={
-                'num_cases': RawSQL.num_cases,
-                'num_runs': RawSQL.num_runs,
-                'num_children': RawSQL.num_plans,
-            })
-            tps = order_plan_queryset(tps, order_by, asc)
-    else:
-        # Set search active plans only by default
-        # I wish to use 'default' argument, as the same as in ModelForm
-        # But it does not seem to work
-        search_form = SearchPlanForm(initial={'is_active': True})
+                # The cleanest way I can find to get it into one query is to
+                # use QuerySet.extra()
+                # See http://docs.djangoproject.com/en/dev/ref/models/querysets
+                .extra(select={
+                    'cases_count': RawSQL.num_cases,
+                    'runs_count': RawSQL.num_runs,
+                    'children_count': RawSQL.num_plans,
+                })
 
-    if request.GET.get('action') == 'clone_case':
-        template_name = 'case/clone_select_plan.html'
-        tps = tps.order_by('name')
-
-    if request.GET.get('t') == 'ajax':
-        data = [
-            dict(
-                pk=plan.pk,
-                name=plan.name,
-                is_active=plan.is_active,
-                parent_id=plan.parent_id,
-                num_cases=plan.num_cases,
-                num_runs=plan.num_runs,
-                num_children=plan.num_children,
-                get_url_path=plan.get_absolute_url(),
+                .order_by('-create_date')
             )
-            for plan in tps
+
+        return search_form, plans
+
+    def get_context_data(self, **kwargs):
+        context = super().get_context_data(**kwargs)
+        context['search_plan_form'], context['plans'] = self.filter_plans()
+        return context
+
+
+class SearchPlansView(SimplePlansFilterView):
+    """Used to filter test plans"""
+
+    SUB_MODULE_NAME = 'plans'
+    template_name = 'plan/all.html'
+
+    def get(self, request, *args, **kwargs):
+        context = self.get_context_data(**kwargs)
+        return self.render_to_response(context)
+
+    def get_context_data(self, **kwargs):
+        context = super().get_context_data(**kwargs)
+        context.update({
+            'module': MODULE_NAME,
+            'sub_module': self.SUB_MODULE_NAME,
+            'object_list': context['plans'][0:20],
+            'plans_count': context['plans'].count(),
+        })
+        return context
+
+
+class SearchPlansPagesView(SimplePlansFilterView):
+
+    template_name = 'plan/common/json_plans.txt'
+    column_names = [
+        '', 'plan_id', 'name', 'author__username', 'owner__username',
+        'product', 'product_version', 'type', 'cases_count', 'runs_count', ''
+    ]
+
+    def get(self, request, *args, **kwargs):
+        _, plans = self.filter_plans()
+        dt = DataTableResult(request.GET, plans, self.column_names)
+        data = dt.get_response_data()
+        resp_data = get_template(self.template_name).render(data, request)
+        return JsonResponse(json.loads(resp_data))
+
+
+class FilterPlansForTreeView(SimplePlansFilterView):
+    """Filter plans for displaying plans tree view"""
+
+    def get(self, request, *args, **kwargs):
+        _, plans = self.filter_plans()
+        data = [
+            {
+                'pk': plan.pk,
+                'name': plan.name,
+                'is_active': plan.is_active,
+                'parent_id': plan.parent_id,
+                'num_cases': plan.cases_count,
+                'num_runs': plan.runs_count,
+                'num_children': plan.children_count,
+                'get_url_path': plan.get_absolute_url(),
+            }
+            for plan in plans
         ]
         return JsonResponse(data, safe=False)
 
-    if request.GET.get('t') == 'html':
-        if request.GET.get('f') == 'preview':
-            template_name = 'plan/preview.html'
-
-    query_url = remove_from_request_path(request, 'order_by')
-    if asc:
-        query_url = remove_from_request_path(query_url, 'asc')
-    else:
-        query_url = '%s&asc=True' % query_url
-    page_type = request.GET.get('page_type', 'pagination')
-    query_url_page_type = remove_from_request_path(request, 'page_type')
-    if query_url_page_type:
-        query_url_page_type = remove_from_request_path(query_url_page_type, 'page')
-
-    context_data = {
-        'module': MODULE_NAME,
-        'sub_module': SUB_MODULE_NAME,
-        'test_plans': tps,
-        'query_result': query_result,
-        'search_plan_form': search_form,
-        'query_url': query_url,
-        'query_url_page_type': query_url_page_type,
-        'page_type': page_type
-    }
-    return render(request, template_name, context=context_data)
-
-
-def get_number_of_plans_cases(plan_ids):
-    """Get the number of cases related to each plan
-
-    Arguments:
-    - plan_ids: a tuple or list of TestPlans' id
-
-    Return value:
-    Return value is an dict object, where key is plan_id and the value is the
-    total count.
-    """
-    qs = TestCasePlan.objects.filter(plan__in=plan_ids)
-    qs = qs.values('plan').annotate(
-        total_count=Count('pk')).order_by('-plan')
-    return {item['plan']: item['total_count'] for item in qs}
-
-
-def get_number_of_plans_runs(plan_ids):
-    """Get the number of runs related to each plan
-
-    Arguments:
-    - plan_ids: a tuple or list of TestPlans' id
-
-    Return value:
-    Return value is an dict object, where key is plan_id and the value is the
-    total count.
-    """
-    qs = TestRun.objects.filter(plan__in=plan_ids)
-    qs = qs.values('plan').annotate(
-        total_count=Count('pk')).order_by('-plan')
-    return {item['plan']: item['total_count'] for item in qs}
-
-
-def get_number_of_children_plans(plan_ids):
-    """Get the number of children plans related to each plan
-
-    Arguments:
-    - plan_ids: a tuple or list of TestPlans' id
-
-    Return value:
-    Return value is an dict object, where key is plan_id and the value is the
-    total count.
-    """
-    qs = TestPlan.objects.filter(parent__in=plan_ids)
-    qs = qs.values('parent').annotate(
-        total_count=Count('parent')).order_by('-parent')
-    return {item['parent']: item['total_count'] for item in qs}
-
-
-def calculate_stats_for_testplans(plans):
-    """Attach the number of cases and runs for each TestPlan
-
-    Arguments:
-    - plans: the queryset of TestPlans
-
-    Return value:
-    A list of TestPlans, each of which is attached the statistics which is
-    with prefix cal meaning calculation result.
-    """
-    plan_ids = [plan.pk for plan in plans]
-    cases_counts = get_number_of_plans_cases(plan_ids)
-    runs_counts = get_number_of_plans_runs(plan_ids)
-    children_counts = get_number_of_children_plans(plan_ids)
-
-    # Attach calculated statistics to each object of TestPlan
-    for plan in plans:
-        setattr(plan, 'cal_cases_count', cases_counts.get(plan.pk, 0))
-        setattr(plan, 'cal_runs_count', runs_counts.get(plan.pk, 0))
-        setattr(plan, 'cal_children_count', children_counts.get(plan.pk, 0))
-
-    return plans
-
-
-@require_GET
-def ajax_search(request, template_name='plan/common/json_plans.txt'):
-    """Display all testplans"""
-    # Define the default sub module
-
-    # If it's not a search the page will be blank
-    tps = TestPlan.objects.none()
-    # if it's a search request the page will be fill
-    if list(request.GET.items()):
-        search_form = SearchPlanForm(request.GET)
-        if request.GET.get('product'):
-            search_form.populate(product_id=request.GET['product'])
-        else:
-            search_form.populate()
-        if search_form.is_valid():
-            # Detemine the query is the user's plans and change the sub
-            # module value
-            author = request.GET.get('author__email__startswith')
-            if author and len(search_form.changed_data) == 1:
-                if request.user.is_authenticated:
-                    if author == request.user.username or author == request.user.email:
-                        tps = TestPlan.objects.filter(
-                            Q(author__email__startswith=author) |
-                            Q(owner__email__startswith=author)
-                        ).distinct()
-            else:
-                tps = TestPlan.list(search_form.cleaned_data)
-                tps = tps.select_related('author', 'owner', 'type', 'product')
-
-    # columnIndexNameMap is required for correct sorting behavior, 5 should
-    # be product, but we use run.build.product
-    column_names = [
-        '',
-        'plan_id',
-        'name',
-        'author__username',
-        'owner__username',
-        'product',
-        'product_version',
-        'type',
-        'num_cases',
-        'num_runs',
-        ''
-    ]
-    return ajax_response(request, tps, column_names,
-                         'plan/common/json_plans.txt')
-
-
-def ajax_response(request, queryset, column_names, template_name):
-    """json template for the ajax request for searching"""
-    dt = DataTableResult(request.GET, queryset, column_names)
-
-    data = dt.get_response_data()
-    data['querySet'] = calculate_stats_for_testplans(data['querySet'])
-
-    # prepare the JSON with the response, consider using :
-    # from django.template.defaultfilters import escapejs
-    t = get_template(template_name)
-    resp_data = t.render(data, request)
-    return JsonResponse(json.loads(resp_data))
-
 
 def get(request, plan_id, slug=None, template_name='plan/get.html'):
     """Display the plan details."""
