@@ -20,33 +20,35 @@ from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
 from django.urls import reverse
 from django.db import transaction
-from django.db.models import Count
+from django.db.models import Count, QuerySet
 from django.db.models import Q
 from django.http import HttpResponse, HttpResponseRedirect, Http404
 from django.http import JsonResponse
 from django.shortcuts import get_object_or_404
 from django.shortcuts import render
 from django.template.loader import get_template
+from django.utils.encoding import smart_str
 from django.views.decorators.csrf import csrf_protect
 from django.views.decorators.http import require_GET
 from django.views.decorators.http import require_POST
 from django.views.decorators.http import require_http_methods
-from django.views.generic import RedirectView
-from django.views.generic.base import TemplateView
-from django.views.generic.base import View
+from django.views.generic import RedirectView, FormView
+from django.views.generic.base import TemplateView, View
 
 from django_comments.models import Comment
-from tcms.core.responses import JsonResponseBadRequest, JsonResponseForbidden
+
+from tcms.core.raw_sql import RawSQL
+from tcms.core.responses import JsonResponseBadRequest
 
 from tcms.issuetracker.models import Issue
 from tcms.issuetracker.models import IssueTracker
 from tcms.issuetracker.services import find_service
 from tcms.core.tcms_router import connection
-from tcms.core.utils import clean_request
+from tcms.core.utils import clean_request, form_error_messages_to_list
 from tcms.core.utils import DataTableResult
 from tcms.core.utils import format_timedelta
 from tcms.core.views import prompt
-from tcms.management.models import Priority, TCMSEnvValue, TestTag, TCMSEnvGroup
+from tcms.management.models import Priority, TestTag, TCMSEnvGroup
 from tcms.search.forms import RunForm
 from tcms.search.query import SmartDjangoQuery
 from tcms.testcases.models import TestCase
@@ -55,7 +57,7 @@ from tcms.testcases.views import get_selected_testcases
 from tcms.testplans.models import TestPlan
 from tcms.testruns.data import stats_caseruns_status
 from tcms.testruns.data import TestCaseRunDataMixin
-from tcms.testruns.forms import MulitpleRunsCloneForm, PlanFilterRunForm
+from tcms.testruns.forms import MulitpleRunsCloneForm, PlanFilterRunForm, RunAndEnvValueForm, ChangeRunEnvValueForm
 from tcms.testruns.forms import NewRunForm, SearchRunForm, EditRunForm, RunCloneForm
 from tcms.testruns.helpers.serializer import TCR2File
 from tcms.testruns.models import TestRun, TestCaseRun, TestCaseRunStatus, TCMSEnvRunValueMap
@@ -274,7 +276,7 @@ def delete(request, run_id):
 
 
 @require_GET
-def all(request, template_name='run/all.html'):
+def all(request):
     """Read the test runs from database and display them."""
     SUB_MODULE_NAME = "runs"
 
@@ -291,13 +293,60 @@ def all(request, template_name='run/all.html'):
     else:
         search_form = SearchRunForm()
 
-    context_data = {
+    return render(request, 'run/all.html', context={
         'module': MODULE_NAME,
         'sub_module': SUB_MODULE_NAME,
         'query_result': query_result,
         'search_form': search_form,
-    }
-    return render(request, template_name, context=context_data)
+    })
+
+
+@require_GET
+def search_runs(request):
+    """Search test runs"""
+    search_form = SearchRunForm(request.GET)
+    product_id = request.GET.get('product')
+    search_form.populate(product_id=int(product_id) if product_id else None)
+
+    runs = TestRun.objects.none()
+
+    if search_form.is_valid():
+        runs = (
+            TestRun
+            .list(search_form.cleaned_data)
+            .select_related('manager', 'default_tester', 'build', 'plan',
+                            'build__product')
+            .only('run_id', 'summary', 'manager__username',
+                  'default_tester__id', 'default_tester__username',
+                  'plan__name', 'build__product__name',
+                  'stop_date', 'product_version__value')
+            .extra(select={'cases_count': RawSQL.total_num_caseruns})
+        )
+
+    column_names = [
+        '', 'run_id', 'summary', 'manager__username',
+        'default_tester__username', 'build__product__name',
+        'product_version__value', 'env_groups', 'cases_count', 'stop_date',
+        'completed',
+    ]
+
+    dt = DataTableResult(request.GET, runs, column_names,
+                         default_order_key='-pk')
+    response_data = dt.get_response_data()
+    calculate_associated_data(response_data['querySet'])
+
+    if 'sEcho' in request.GET:
+        resp_data = (get_template('run/common/json_runs.txt')
+                     .render(response_data, request))
+        return JsonResponse(json.loads(resp_data))
+    else:
+        return render(request, 'run/all.html', context={
+            'module': MODULE_NAME,
+            'sub_module': 'runs',
+            'object_list': response_data['querySet'],
+            'search_form': search_form,
+            'total_count': runs.count()
+        })
 
 
 def run_queryset_from_querystring(querystring):
@@ -336,7 +385,7 @@ def load_runs_of_one_plan(request, plan_id,
     data, it accepts field lookup parameters collected from the filter panel
     in the UI.
     """
-    column_names = {
+    column_names = [
         '',
         'run_id',
         'summary',
@@ -348,7 +397,7 @@ def load_runs_of_one_plan(request, plan_id,
         'total_num_caseruns',
         'failure_caseruns_percent',
         'successful_caseruns_percent',
-    }
+    ]
 
     tp = TestPlan.objects.get(plan_id=plan_id)
     form = PlanFilterRunForm(request.GET)
@@ -420,121 +469,65 @@ def load_runs_of_one_plan(request, plan_id,
     return JsonResponse(json.loads(resp_data))
 
 
-@require_GET
-def ajax_search(request, template_name='run/common/json_runs.txt'):
-    """Response request to search test runs from Search Runs"""
-    search_form = SearchRunForm(request.GET)
-    if request.GET.get('product'):
-        search_form.populate(product_id=request.GET['product'])
-    else:
-        search_form.populate()
+def calculate_associated_data(runs: QuerySet) -> None:
+    """Calculate associated data and set to each run in place
 
-    if search_form.is_valid():
-        trs = TestRun.list(search_form.cleaned_data)
-        trs = trs.select_related(
-            'manager',
-            'default_tester',
-            'build',
-            'plan',
-            'build__product'
-        ).only(
-            'run_id',
-            'summary',
-            'manager__username',
-            'default_tester__id',
-            'default_tester__username',
-            'plan__name',
-            'build__product__name',
-            'stop_date',
-            'product_version__value'
-        )
+    The associated data include:
 
-        column_names = [
-            '',
-            'run_id',
-            'summary',
-            'manager__username',
-            'default_tester__username',
-            'plan',
-            'build__product__name',
-            'product_version',
-            'env_groups',
-            'total_num_caseruns',
-            'stop_date',
-            'completed',
-        ]
+    * completed progress of each test run
+    * the environment of each test run
+    """
+    run_ids = [run.pk for run in runs]
+    qs = (TestCaseRun.objects
+          .filter(case_run_status=TestCaseRunStatus.name_to_id('FAILED'),
+                  run__in=run_ids)
+          .values('run', 'case_run_status')
+          .annotate(count=Count('pk'))
+          .order_by('run', 'case_run_status'))
+    failure_subtotal = magic_convert(qs, key_name='run', value_name='count')
 
-        dt = DataTableResult(request.GET, trs, column_names)
-        response_data = dt.get_response_data()
-        searched_runs = response_data['querySet']
+    completed_status_ids = TestCaseRunStatus.completed_status_ids()
+    qs = (TestCaseRun.objects
+          .filter(case_run_status__in=completed_status_ids, run__in=run_ids)
+          .values('run', 'case_run_status')
+          .annotate(count=Count('pk'))
+          .order_by('run', 'case_run_status'))
+    completed_subtotal = {
+        run_id: sum((item['count'] for item in stats_rows))
+        for run_id, stats_rows
+        in itertools.groupby(qs.iterator(), key=itemgetter('run'))
+    }
 
-        # Get associated statistics data
-        run_ids = [run.pk for run in searched_runs]
-        qs = TestCaseRun.objects.filter(
-            case_run_status=TestCaseRunStatus.name_to_id('FAILED'),
-            run__in=run_ids
-        ).values(
-            'run', 'case_run_status'
-        ).annotate(
-            count=Count('pk')
-        ).order_by('run', 'case_run_status')
-        failure_subtotal = magic_convert(qs, key_name='run', value_name='count')
+    qs = (TestCaseRun.objects
+          .filter(run__in=run_ids)
+          .values('run')
+          .annotate(cases_count=Count('case')))
+    cases_subtotal = magic_convert(qs, key_name='run', value_name='cases_count')
 
-        completed_status_ids = TestCaseRunStatus.completed_status_ids()
-        qs = TestCaseRun.objects.filter(
-            case_run_status__in=completed_status_ids,
-            run__in=run_ids
-        ).values(
-            'run', 'case_run_status'
-        ).annotate(
-            count=Count('pk')
-        ).order_by('run', 'case_run_status')
-        completed_subtotal = {
-            run_id: sum((item['count'] for item in stats_rows))
-            for run_id, stats_rows
-            in itertools.groupby(qs.iterator(), key=itemgetter('run'))
+    # Relative env groups to runs
+    result = (TCMSEnvGroup.objects
+              .filter(plans__run__in=run_ids)
+              .values('plans__run', 'name'))
+    runs_env_groups = {item['plans__run']: item['name'] for item in result}
+
+    for run in runs:
+        run_id = run.pk
+        cases_count = cases_subtotal.get(run_id, 0)
+        if cases_count:
+            completed_percent = completed_subtotal.get(run_id, 0) * 1.0 / cases_count * 100
+            failure_percent = failure_subtotal.get(run_id, 0) * 1.0 / cases_count * 100
+        else:
+            completed_percent = failure_percent = 0
+
+        run.associated_data = {
+            'stats': {
+                'cases': cases_count,
+                'completed_percent': completed_percent,
+                'failure_percent': failure_percent,
+            },
+            'env_group': runs_env_groups.get(run_id),
         }
 
-        qs = TestCaseRun.objects.filter(
-            run__in=run_ids
-        ).values('run').annotate(
-            cases_count=Count('case')
-        )
-        cases_subtotal = magic_convert(qs, key_name='run', value_name='cases_count')
-
-        # Relative env groups to runs
-        result = (TCMSEnvGroup.objects.filter(plans__run__in=run_ids)
-                                      .values('plans__run', 'name'))
-        runs_env_groups = {item['plans__run']: item['name'] for item in result}
-
-        for run in searched_runs:
-            run_id = run.pk
-            cases_count = cases_subtotal.get(run_id, 0)
-            if cases_count:
-                completed_percent = completed_subtotal.get(run_id, 0) * 1.0 / cases_count * 100
-                failure_percent = failure_subtotal.get(run_id, 0) * 1.0 / cases_count * 100
-            else:
-                completed_percent = failure_percent = 0
-
-            run.associated_data = {
-                'stats': {
-                    'cases': cases_count,
-                    'completed_percent': completed_percent,
-                    'failure_percent': failure_percent,
-                },
-                'env_group': runs_env_groups.get(run_id),
-            }
-    else:
-        response_data = {
-            'sEcho': int(request.GET.get('sEcho', 0)),
-            'iTotalRecords': 0,
-            'iTotalDisplayRecords': 0,
-            'runs': TestRun.objects.none(),
-        }
-
-    resp_data = get_template(template_name).render(response_data, request)
-    return JsonResponse(json.loads(resp_data))
-
 
 def open_run_get_case_runs(request, run):
     """Prepare for case runs list in a TestRun page
@@ -1282,93 +1275,64 @@ def export(request, run_id):
     return response
 
 
-@require_GET
-def env_value(request):
-    """Run environment property edit function"""
-    trs = TestRun.objects.filter(run_id__in=request.GET.getlist('run_id'))
+class AddEnvValueToRunView(PermissionRequiredMixin, FormView):
+    """Add env value to test runs"""
 
-    class RunEnvActions:
-        def __init__(self, requet, trs):
-            self.__all__ = ['add', 'remove', 'change']
-            self.request = request
-            self.trs = trs
+    form_class = RunAndEnvValueForm
+    permission_required = 'testruns.add_tcmsenvrunvaluemap'
+    raise_exception = True
 
-        def has_no_perm(self, perm):
-            if not self.request.user.has_perm('testruns.' + perm + '_tcmsenvrunvaluemap'):
-                return {'message': 'Permission denied - %s' % perm}
+    def form_valid(self, form):
+        for run in form.cleaned_data['runs']:
+            run.add_env_value(form.cleaned_data['env_value'])
 
-            return False
+        fragment = render(self.request, 'run/get_environment.html', context={
+            'test_run': form.cleaned_data['runs'][0],
+            'is_ajax': True
+        })
+        return JsonResponse({'fragment': smart_str(fragment.content)})
 
-        def get_env_value(self, env_value_id):
-            return TCMSEnvValue.objects.get(id=env_value_id)
+    def form_invalid(self, form):
+        return JsonResponseBadRequest({
+            'message': form_error_messages_to_list(form.errors)
+        })
 
-        def add(self):
-            chk_perm = self.has_no_perm('add')
 
-            if chk_perm:
-                return JsonResponseForbidden(chk_perm)
+class DeleteRunEnvValueView(PermissionRequiredMixin, FormView):
+    """Delete a test run's env value"""
 
-            env_value_id = int(request.GET.get('env_value_id'))
-            try:
-                value = self.get_env_value(env_value_id)
-            except ObjectDoesNotExist:
-                return JsonResponseBadRequest({
-                    'message': f'Environment value id {env_value_id} does not exist.'
-                })
+    form_class = RunAndEnvValueForm
+    permission_required = 'testruns.delete_tcmsenvrunvaluemap'
+    raise_exception = True
 
-            for tr in self.trs:
-                tr.add_env_value(env_value=value)
+    def form_valid(self, form):
+        for run in form.cleaned_data['runs']:
+            run.remove_env_value(form.cleaned_data['env_value'])
+        return JsonResponse({})
 
-            fragment = render(request, 'run/get_environment.html', context={
-                'test_run': self.trs[0],
-                'is_ajax': True
-            })
+    def form_invalid(self, form):
+        return JsonResponseBadRequest({
+            'message': form_error_messages_to_list(form.errors)
+        })
 
-            if isinstance(fragment.content, bytes):
-                s = fragment.content.decode('utf-8')
-            else:
-                s = fragment.content
 
-            return JsonResponse({'fragment': s})
+class ChangeRunEnvValueView(PermissionRequiredMixin, FormView):
+    """Change a test run's env value"""
 
-        def remove(self):
-            chk_perm = self.has_no_perm('delete')
-            if chk_perm:
-                return JsonResponseForbidden(chk_perm)
+    form_class = ChangeRunEnvValueForm
+    permission_required = 'testruns.change_tcmsenvrunvaluemap'
+    raise_exception = True
 
-            try:
-                for tr in self.trs:
-                    tr.remove_env_value(env_value=self.get_env_value(
-                        request.GET.get('env_value_id')
-                    ))
-            except Exception:
-                pass
+    def form_valid(self, form):
+        for run in form.cleaned_data['runs']:
+            run.remove_env_value(form.cleaned_data['old_env_value'])
+            run.add_env_value(form.cleaned_data['new_env_value'])
+        return JsonResponse({})
 
-            return JsonResponse({})
-
-        def change(self):
-            chk_perm = self.has_no_perm('change')
-            if chk_perm:
-                return JsonResponseForbidden(chk_perm)
-
-            for tr in self.trs:
-                tr.remove_env_value(env_value=self.get_env_value(
-                    request.GET.get('old_env_value_id')
-                ))
-
-                tr.add_env_value(env_value=self.get_env_value(
-                    request.GET.get('new_env_value_id')
-                ))
-
-            return JsonResponse({})
-
-    run_env_actions = RunEnvActions(request, trs)
-
-    if request.GET.get('a') not in run_env_actions.__all__:
-        return JsonResponseBadRequest({'message': 'Unrecognizable actions'})
-
-    func = getattr(run_env_actions, request.GET['a'])
-    return func()
+    def form_invalid(self, form):
+        return JsonResponseBadRequest({
+            'message': form_error_messages_to_list(form.errors)
+        })
 
 
 def caseruns(request, templ='report/caseruns.html'):
