@@ -57,7 +57,6 @@ class PlanTests(HelperAssertions, test.TestCase):
             author=cls.user,
             product=cls.product,
             type=cls.plan_type)
-        cls.email_settings = f.TestPlanEmailSettingsFactory(plan=cls.test_plan)
         cls.plan_id = cls.test_plan.pk
 
     def test_open_plans_search(self):
@@ -492,6 +491,8 @@ class TestCloneView(BasePlanCase):
                            link_cases=True, copy_cases=None,
                            maintain_case_orignal_author=None,
                            keep_case_default_tester=None):
+        self.assertIsNotNone(cloned_plan.email_settings)
+
         self.assertEqual(f'Copy of {original_plan.name}', cloned_plan.name)
         self.assertEqual(Product.objects.get(pk=self.product.pk), cloned_plan.product)
         self.assertEqual(Version.objects.get(pk=self.version.pk), cloned_plan.product_version)
@@ -653,7 +654,7 @@ class TestCloneView(BasePlanCase):
             self.verify_cloned_plan(origin_plan, cloned_plan)
 
 
-class TestAJAXSearch(BasePlanCase):
+class TestPlansPagesView(BasePlanCase):
     """Test ajax_search view method"""
 
     @classmethod
@@ -669,7 +670,7 @@ class TestAJAXSearch(BasePlanCase):
 
         # So far, each test has 26 plans
 
-        cls.search_url = reverse('plans-ajax-search')
+        cls.search_url = reverse('plans-pages')
 
         # By default, search active plans. Search by other fields if needed,
         # copy this dict and add other fields.
@@ -692,16 +693,6 @@ class TestAJAXSearch(BasePlanCase):
             'bSortable_4': 'true',
         }
 
-    def test_emtpy_plans(self):
-        response = self.client.get(self.search_url, {})
-
-        data = json.loads(response.content)
-
-        self.assertEqual(0, data['sEcho'])
-        self.assertEqual(0, data['iTotalRecords'])
-        self.assertEqual(0, data['iTotalDisplayRecords'])
-        self.assertEqual([], data['aaData'])
-
     def test_get_first_page_order_by_pk(self):
         search_data = self.search_data.copy()
 
